/*  Misfit Model 3D
 * 
 *  Copyright (c) 2004 Kevin Worcester
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
 *  USA.
 *
 *  See the COPYING file for full license text.
 */


#include "qtmain.h"
#include "stdcmds.h"
#include "stdtools.h"
#include "stdfilters.h"
#include "stdtexfilters.h"
#include "pluginmgr.h"
#include "cmdline.h"
#include "3dmprefs.h"
#include "model.h"
#include "log.h"
#include "cmdmgr.h"
#include "decalmgr.h"
#include "filtermgr.h"
#include "texmgr.h"

#include <signal.h>

void segfault_handler( int sig )
{
   fprintf( stderr, "Segfault.  Exiting...\n" );
   exit( 0 );
}

int main( int argc, char * argv[] )
{
   int rval = 0;
   log_profile_init( "profile_data.txt" );

   signal( SIGSEGV, segfault_handler );

   {
      LOG_PROFILE();

      {
         LOG_PROFILE_STR( "Initialize" );

         init_prefs();
         init_cmdline( argc, argv );

         init_std_filters();
         init_std_texture_filters();
         init_std_tools();
         init_std_cmds( CommandManager::getInstance() );

         init_plugins();
      }

      if ( cmdline_runcommand )
      {
         rval = cmdline_command( argc, argv );
      }
      if ( cmdline_runui )
      {
         rval = ui_init( argc, argv );
      }

      {
         LOG_PROFILE_STR( "Uninitialize" );

         shutdown_cmdline();

         model_show_alloc_stats();

         DecalManager::release();
         CommandManager::release();
         FilterManager::release();
         TextureManager::release();
         PluginManager::release();

         model_free_primitives();
         model_show_alloc_stats();

         prefs_save();
      }
   }

   log_profile_shutdown();

   return rval;
}
